# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "sig update",
)
class Update(AAZCommand):
    """Update a share image gallery.

    :example: Enable gallery to be shared to subscription or tenant
        az sig update --resource-group myResourceGroup --gallery-name myGallery --permissions groups

    :example: Update gallery from private to community
        az sig update -g myResourceGroup --gallery-name myGallery --permissions Community --publisher-uri myPublisherUri --publisher-email myPublisherEmail --eula myEula --public-name-prefix myPublicNamePrefix
    """

    _aaz_info = {
        "version": "2021-10-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.compute/galleries/{}", "2021-10-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    AZ_SUPPORT_GENERIC_UPDATE = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.gallery_name = AAZStrArg(
            options=["-r", "--gallery-name"],
            help="The name of the Shared Image Gallery to be deleted.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )

        # define Arg Group "CommunityGalleryInfo"

        _args_schema = cls._args_schema
        _args_schema.eula = AAZStrArg(
            options=["--eula"],
            arg_group="CommunityGalleryInfo",
            help="Community gallery publisher eula",
            nullable=True,
        )
        _args_schema.public_name_prefix = AAZStrArg(
            options=["--public-name-prefix"],
            arg_group="CommunityGalleryInfo",
            help="Community gallery public name prefix",
            nullable=True,
        )
        _args_schema.publisher_contact = AAZStrArg(
            options=["--publisher-email", "--publisher-contact"],
            arg_group="CommunityGalleryInfo",
            help="Community gallery publisher contact email",
            nullable=True,
        )
        _args_schema.publisher_uri = AAZStrArg(
            options=["--publisher-uri"],
            arg_group="CommunityGalleryInfo",
            help="Community gallery publisher uri",
            nullable=True,
        )

        # define Arg Group "Gallery"

        _args_schema = cls._args_schema
        _args_schema.location = AAZResourceLocationArg(
            arg_group="Gallery",
            help="Resource location",
            fmt=AAZResourceLocationArgFormat(
                resource_group_arg="resource_group",
            ),
        )

        # define Arg Group "Properties"

        # define Arg Group "SharingProfile"

        _args_schema = cls._args_schema
        _args_schema.permissions = AAZStrArg(
            options=["--permissions"],
            arg_group="SharingProfile",
            help="This property allows you to specify the permission of sharing gallery.",
            nullable=True,
            enum={"Community": "Community", "Groups": "Groups", "Private": "Private"},
        )

        # define Arg Group "SoftDeletePolicy"

        _args_schema = cls._args_schema
        _args_schema.soft_delete = AAZBoolArg(
            options=["--soft-delete"],
            arg_group="SoftDeletePolicy",
            help="Enable soft-deletion for resources in this gallery, allowing them to be recovered within retention time.",
            nullable=True,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.GalleriesGet(ctx=self.ctx)()
        self.pre_instance_update(self.ctx.vars.instance)
        self.InstanceUpdateByJson(ctx=self.ctx)()
        self.InstanceUpdateByGeneric(ctx=self.ctx)()
        self.post_instance_update(self.ctx.vars.instance)
        yield self.GalleriesCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    @register_callback
    def pre_instance_update(self, instance):
        pass

    @register_callback
    def post_instance_update(self, instance):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class GalleriesGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/galleries/{galleryName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "galleryName", self.ctx.args.gallery_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2021-10-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _UpdateHelper._build_schema_gallery_read(cls._schema_on_200)

            return cls._schema_on_200

    class GalleriesCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200, 201]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/galleries/{galleryName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "galleryName", self.ctx.args.gallery_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2021-10-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=self.ctx.vars.instance,
            )

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()
            _UpdateHelper._build_schema_gallery_read(cls._schema_on_200_201)

            return cls._schema_on_200_201

    class InstanceUpdateByJson(AAZJsonInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance(self.ctx.vars.instance)

        def _update_instance(self, instance):
            _instance_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=instance,
                typ=AAZObjectType
            )
            _builder.set_prop("location", AAZStrType, ".location", typ_kwargs={"flags": {"required": True}})
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("sharingProfile", AAZObjectType)
                properties.set_prop("softDeletePolicy", AAZObjectType)

            sharing_profile = _builder.get(".properties.sharingProfile")
            if sharing_profile is not None:
                sharing_profile.set_prop("communityGalleryInfo", AAZObjectType)
                sharing_profile.set_prop("permissions", AAZStrType, ".permissions")

            community_gallery_info = _builder.get(".properties.sharingProfile.communityGalleryInfo")
            if community_gallery_info is not None:
                community_gallery_info.set_prop("eula", AAZStrType, ".eula")
                community_gallery_info.set_prop("publicNamePrefix", AAZStrType, ".public_name_prefix")
                community_gallery_info.set_prop("publisherContact", AAZStrType, ".publisher_contact")
                community_gallery_info.set_prop("publisherUri", AAZStrType, ".publisher_uri")

            soft_delete_policy = _builder.get(".properties.softDeletePolicy")
            if soft_delete_policy is not None:
                soft_delete_policy.set_prop("isSoftDeleteEnabled", AAZBoolType, ".soft_delete")

            return _instance_value

    class InstanceUpdateByGeneric(AAZGenericInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance_by_generic(
                self.ctx.vars.instance,
                self.ctx.generic_update_args
            )


class _UpdateHelper:
    """Helper class for Update"""

    _schema_gallery_read = None

    @classmethod
    def _build_schema_gallery_read(cls, _schema):
        if cls._schema_gallery_read is not None:
            _schema.id = cls._schema_gallery_read.id
            _schema.location = cls._schema_gallery_read.location
            _schema.name = cls._schema_gallery_read.name
            _schema.properties = cls._schema_gallery_read.properties
            _schema.tags = cls._schema_gallery_read.tags
            _schema.type = cls._schema_gallery_read.type
            return

        cls._schema_gallery_read = _schema_gallery_read = AAZObjectType()

        gallery_read = _schema_gallery_read
        gallery_read.id = AAZStrType(
            flags={"read_only": True},
        )
        gallery_read.location = AAZStrType(
            flags={"required": True},
        )
        gallery_read.name = AAZStrType(
            flags={"read_only": True},
        )
        gallery_read.properties = AAZObjectType(
            flags={"client_flatten": True},
        )
        gallery_read.tags = AAZDictType()
        gallery_read.type = AAZStrType(
            flags={"read_only": True},
        )

        properties = _schema_gallery_read.properties
        properties.description = AAZStrType()
        properties.identifier = AAZObjectType()
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.sharing_profile = AAZObjectType(
            serialized_name="sharingProfile",
        )
        properties.sharing_status = AAZObjectType(
            serialized_name="sharingStatus",
            flags={"read_only": True},
        )
        properties.soft_delete_policy = AAZObjectType(
            serialized_name="softDeletePolicy",
        )

        identifier = _schema_gallery_read.properties.identifier
        identifier.unique_name = AAZStrType(
            serialized_name="uniqueName",
            flags={"read_only": True},
        )

        sharing_profile = _schema_gallery_read.properties.sharing_profile
        sharing_profile.community_gallery_info = AAZObjectType(
            serialized_name="communityGalleryInfo",
        )
        sharing_profile.groups = AAZListType(
            flags={"read_only": True},
        )
        sharing_profile.permissions = AAZStrType()

        community_gallery_info = _schema_gallery_read.properties.sharing_profile.community_gallery_info
        community_gallery_info.community_gallery_enabled = AAZBoolType(
            serialized_name="communityGalleryEnabled",
            flags={"read_only": True},
        )
        community_gallery_info.eula = AAZStrType()
        community_gallery_info.public_name_prefix = AAZStrType(
            serialized_name="publicNamePrefix",
        )
        community_gallery_info.public_names = AAZListType(
            serialized_name="publicNames",
            flags={"read_only": True},
        )
        community_gallery_info.publisher_contact = AAZStrType(
            serialized_name="publisherContact",
        )
        community_gallery_info.publisher_uri = AAZStrType(
            serialized_name="publisherUri",
        )

        public_names = _schema_gallery_read.properties.sharing_profile.community_gallery_info.public_names
        public_names.Element = AAZStrType()

        groups = _schema_gallery_read.properties.sharing_profile.groups
        groups.Element = AAZObjectType()

        _element = _schema_gallery_read.properties.sharing_profile.groups.Element
        _element.ids = AAZListType()
        _element.type = AAZStrType()

        ids = _schema_gallery_read.properties.sharing_profile.groups.Element.ids
        ids.Element = AAZStrType()

        sharing_status = _schema_gallery_read.properties.sharing_status
        sharing_status.aggregated_state = AAZStrType(
            serialized_name="aggregatedState",
            flags={"read_only": True},
        )
        sharing_status.summary = AAZListType()

        summary = _schema_gallery_read.properties.sharing_status.summary
        summary.Element = AAZObjectType()

        _element = _schema_gallery_read.properties.sharing_status.summary.Element
        _element.details = AAZStrType()
        _element.region = AAZStrType()
        _element.state = AAZStrType(
            flags={"read_only": True},
        )

        soft_delete_policy = _schema_gallery_read.properties.soft_delete_policy
        soft_delete_policy.is_soft_delete_enabled = AAZBoolType(
            serialized_name="isSoftDeleteEnabled",
        )

        tags = _schema_gallery_read.tags
        tags.Element = AAZStrType()

        _schema.id = cls._schema_gallery_read.id
        _schema.location = cls._schema_gallery_read.location
        _schema.name = cls._schema_gallery_read.name
        _schema.properties = cls._schema_gallery_read.properties
        _schema.tags = cls._schema_gallery_read.tags
        _schema.type = cls._schema_gallery_read.type


__all__ = ["Update"]
